﻿using CommunityToolkit.Mvvm.Input;
using LiveCharts;
using Misc;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace AD2021.B2.TestApp
{
    /// <summary>
    /// Window1.xaml 的交互逻辑
    /// </summary>
    public partial class WdTimeGrid : Window
    {
        WdTimeGridVm viewModel;
        public WdTimeGrid()
        {
            InitializeComponent();
            this.Unloaded += WdTimeGrid_Unloaded;
        }

        private void WdTimeGrid_Unloaded(object sender, RoutedEventArgs e)
        {
            if (viewModel != null) {
                viewModel.Dispose();
                viewModel = null;
            }
        }

        [Unity.InjectionMethod]
        public void Init(FlyADBase.FlyAd2021B2 flyAd) 
        {
            viewModel = new WdTimeGridVm();
            viewModel.Init(flyAd);
            this.DataContext = viewModel;
        }
    }

    public class WdTimeGridVm : INotifyPropertyChanged, IDisposable
    {
        public event PropertyChangedEventHandler PropertyChanged;




        public ChartValues<int> Data1s { get; } = new ChartValues<int>();
        public ChartValues<int> Data2s { get; } = new ChartValues<int>();

        public int Value1 { get; set; }

        public int Average1 { get; set; }
        public int Min1 { get; set; }
        public int Max1 { get; set; }
        public int Sigma2_1 { get; set; }

        public int Value2 { get; set; }

        public int Average2 { get; set; }
        public int Min2 { get; set; }
        public int Max2 { get; set; }
        public int Sigma2_2 { get; set; }

        /// <summary>
        /// values容量
        /// </summary>
        public int ValuesCap { get; set; } = 200;
        /// <summary>
        /// 间隔
        /// </summary>
        public int Interval { get; set; } = 100;
        /// <summary>
        /// X轴间隔
        /// </summary>
        public int XStep { get; set; } = 10;

        /// <summary>
        /// Y轴范围
        /// </summary>
        public double YRange { get; set; } = double.NaN;
        public double YMax1 { get; set; } = double.NaN;
        public double YMin1 { get; set; } = double.NaN;
        public double YMax2 { get; set; } = double.NaN;
        public double YMin2 { get; set; } = double.NaN;
        public RelayCommand ClearCmd { get; private set; }
        public RelayCommand SaveCmd { get; private set; }
        List<int> Buf1 = new List<int>();
        List<int> Buf2 = new List<int>();

        List<int> Ad1s = new List<int>();
        List<int> Ad2s = new List<int>();

        FlyADBase.FlyAd2021B2 flyAd;

        public WdTimeGridVm() 
        {
            ClearCmd = new RelayCommand(Clear);
            SaveCmd = new RelayCommand(Save);


        }

        public void Init(FlyADBase.FlyAd2021B2 flyAd)
        {
            this.flyAd = flyAd;
            this.flyAd.TimeGridEvent += FlyAd_TimeGridEvent;

            if (!Load())
            {
                Save();
            }
        }


        private void Clear()
        {
            Ad1s.Clear();
            Ad2s.Clear();
        }


        private void FlyAd_TimeGridEvent(object sender, FlyADBase.TimeGridEventArgs e)
        {
            //确保 Data 只有 2000个数据
            if (Interval <= 0)
            {
                Ad1s.AddRange(e.Ad1s);
                Ad2s.AddRange(e.Ad2s);
                Buf1.Clear();
                Buf2.Clear();
            }
            else
            {
                for (int i = 0; i < e.Ad1s.Count(); i++)
                {
                    Buf1.Add(e.Ad1s[i]);
                    if (Buf1.Count() >= Interval)
                    {
                        Ad1s.Add((int)Buf1.Average());
                        Buf1.Clear();
                    }
                    Buf2.Add(e.Ad2s[i]);
                    if (Buf2.Count() >= Interval)
                    {
                        Ad2s.Add((int)Buf2.Average());
                        Buf2.Clear();
                    }
                }
            }

            if (Ad1s.Count > ValuesCap)
            {
                int removeCnt = Ad1s.Count - ValuesCap;
                Ad1s.RemoveRange(0, removeCnt);
                Ad2s.RemoveRange(0, removeCnt);
            }

            Data1s.Clear();
            Data2s.Clear();

            Data1s.AddRange(Ad1s);
            Data2s.AddRange(Ad2s);

            if (Ad1s.Count() > 10) {
                Value1 = (int)Ad1s.GetRange(Ad1s.Count() - 10, 10).Average();
                Value2 = (int)Ad2s.GetRange(Ad2s.Count() - 10, 10).Average();
            }
            else 
            {
                Value1 = Ad1s.Last();
                Value2 = Ad2s.Last();
            }
            Average1 = (int)Ad1s.Average();
            Max1 = (int)Ad1s.Max();
            Min1 = (int)Ad1s.Min();
            Sigma2_1 = (int)Ad1s.Sigma() * 2;

            Average2 = (int)Ad2s.Average();
            Max2 = (int)Ad2s.Max();
            Min2 = (int)Ad2s.Min();
            Sigma2_2 = (int)Ad2s.Sigma() * 2;

            if (double.IsNaN(YRange) || YRange<10)
            {
                YMax1 = double.NaN;
                YMin1 = double.NaN;
                YMax2 = double.NaN;
                YMin2 = double.NaN;
            }
            else 
            {
                if (IsY1OutOfShowRange()) {
                    YMax1 = Value1 + YRange / 2;
                    YMin1 = Value1 - YRange / 2;
                }

                if (IsY2OutOfShowRange())
                {
                    YMax2 = Value2 + YRange / 2;
                    YMin2 = Value2 - YRange / 2;
                }
            }
            return;
        }

        bool IsY1OutOfShowRange() 
        {
            //超出显示范围
            if (double.IsNaN(YMax1))
                return true;
            if (YMax1 - YMin1 != YRange)
                return true;

            double mid1 = (YMax1 + YMin1) / 2;
            if (Math.Abs(Value1 - mid1) > YRange / 4)
                return true;
            return false;
        }
        bool IsY2OutOfShowRange()
        {
            //超出显示范围
            if (double.IsNaN(YMax2))
                return true;
            if (YMax2 - YMin2 != YRange)
                return true;

            double mid2 = (YMax2 + YMin2) / 2;
            if (Math.Abs(Value2 - mid2) > YRange / 4)
                return true;
            return false;
        }
        public void Dispose()
        {
            flyAd.TimeGridEvent -= FlyAd_TimeGridEvent;
        }


        string filePath = "timegrid.json";
        private void Save()
        {
            WdTimeGridVmJsonDb.Save(filePath, this);
        }
        private bool Load() 
        {
            return WdTimeGridVmJsonDb.Load(filePath, this);
        }
    }

    public class WdTimeGridVmJsonDb
    {
        public int ValuesCap { get; set; } = 200;
        public int Interval { get; set; } = 100;
        public int XStep { get; set; } = 10;

        /// <summary>
        /// Y轴范围
        /// </summary>
        public double YRange { get; set; } = double.NaN;

        public static bool Save(string path, WdTimeGridVm src)
        {

            WdTimeGridVmJsonDb jsonDb = new WdTimeGridVmJsonDb() {
                Interval = src.Interval,
                ValuesCap = src.ValuesCap,
                XStep = src.XStep,
                YRange = src.YRange
            };
            try
            {
                var json = Newtonsoft.Json.JsonConvert.SerializeObject(jsonDb, Newtonsoft.Json.Formatting.Indented);
                File.WriteAllText(path, json);
                return true;
            }
            catch(Exception e)
            {
                return false;
            }
        }
        public static bool Load(string path, WdTimeGridVm src)
        {
            if (!File.Exists(path)) 
            {
                return false;
            }
            try
            {
                string json = File.ReadAllText(path);
                var jsonDb = Newtonsoft.Json.JsonConvert.DeserializeObject<WdTimeGridVmJsonDb>(json);
                src.Interval = jsonDb.Interval;
                src.ValuesCap = jsonDb.ValuesCap;
                src.XStep = jsonDb.XStep;
                src.YRange = jsonDb.YRange;
                return true;
            }
            catch (Exception e) {
                return false;
            }
        }
    }
}
