﻿using GeneralGommunication;
using Misc;
using System;
using System.ComponentModel;

namespace FlyXray
{
    public class FlyXrayUnit : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public bool IsError => core != null ? core.IsError : false;

        public string ErrMsg => core != null ? core.ErrMsg : null;

        public int ErrCnt => core != null ? core.ErrCnt : 0;

        public bool IsConnected => core != null ? core.IsConnected : false;

        public int ID
        {
            get
            {
                return core != null ? core.ID : 0;
            }
            set {
                if (core != null)
                {
                    core.ID = value;
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        public TimeSpan UpdateInterval { get; protected set; } = TimeSpan.FromSeconds(0.5);
        /// <summary>
        /// 当前高压
        /// </summary>
        public UInt32 HvCurrent { get; private set; }

        /// <summary>
        /// 当前管流
        /// </summary>
        public UInt32 IanCurrent { get; private set; }

        public FlyXrayCore Core => core;

        FlyXrayCore core;
        ModbusAsciiAsync modbus;
        ModbusAsciiAsyncConnector connector;
        /// <summary>
        /// 初始化
        /// </summary>
        /// <param name="modbus"></param>
        public void Init(ModbusAsciiAsyncConnector _connector, TimeSpan updateInterval ) 
        {
            connector = _connector;
            modbus = connector.Modbus;
            core = new FlyXrayCore();
            core.Init(modbus);
            core.PropertyChanged += Core_PropertyChanged;

            this.UpdateInterval = updateInterval;
            PollModule.Current.Poll_Config(OnUpdate, UpdateInterval);
        }

        private void Core_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            PropertyChanged?.Invoke(this, e);
        }

        void OnUpdate()
        {
            if (!connector.IsConnected)//还没连上
                return;

            //当modbus任务列表已经有，不再发送
            if (modbus.Contain(ID, nameof(core.GetCurrentValue)))
                return;

            core.GetCurrentValue(GetCurrentValueDelegate, this);
        }
        void GetCurrentValueDelegate(object asyncContext, object retData) 
        {
            var response = retData as GetCurrentValue_Response;
            HvCurrent = response.HvCurrent;
            IanCurrent = response.IanCurrent;
        }
    }
}
